/*
 * Copyright 2006 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.user.client.ui;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;

/**
 * A type of widget that can wrap another widget, hiding the wrapped widget's
 * methods. When added to a panel, a composite behaves exactly as if the widget
 * it wraps had been added.
 * 
 * <p>
 * The composite is useful for creating a single widget out of an aggregate of
 * multiple other widgets contained in a single panel.
 * </p>
 *
 * <h3>Example</h3>
 * {@link com.google.gwt.examples.CompositeExample code}
 */
public abstract class Composite extends Widget {

  private Widget widget;

  protected void onAttach() {
    super.onAttach();
    widget.onAttach();
  }

  protected void onDetach() {
    super.onDetach();
    widget.onDetach();
  }

  /**
   * This override checks to ensure {@link #setWidget(Widget)} has been called.
   */
  public Element getElement() {
    if (widget == null) {
      throw new IllegalStateException("setWidget() was never called in "
        + GWT.getTypeName(this));
    }
    return super.getElement();
  }

  /**
   * Sets the widget to be wrapped by the composite. The wrapped widget must be
   * set before calling any {@link Widget} methods on this object, or adding it
   * to a panel.
   * 
   * @param widget the widget to be wrapped
   */
  protected void setWidget(Widget widget) {
    this.widget = widget;
    setElement(widget.getElement());
  }

  void setParent(Panel parent) {
    super.setParent(parent);
    widget.setParent(parent);
  }
}
